/** Export to JSON */
/**
 * Checks that an element has a non-empty `name` and `value` property.
 * @param  {Element} element  the element to check
 * @return {Bool}             true if the element is an input, false if not
 */
const isValidElement = element => {
  return element.name && element.value;
};

/**
 * Checks if an element’s value can be saved (e.g. not an unselected checkbox).
 * @param  {Element} element  the element to check
 * @return {Boolean}          true if the value should be added, false if not
 */
const isValidValue = element => {
  return (!['checkbox', 'radio'].includes(element.type) || element.checked);
};

/**
 * Checks if an input is a checkbox, because checkboxes allow multiple values.
 * @param  {Element} element  the element to check
 * @return {Boolean}          true if the element is a checkbox, false if not
 */
const isCheckbox = element => element.type === 'checkbox';

/**
 * Checks if an input is a `select` with the `multiple` attribute.
 * @param  {Element} element  the element to check
 * @return {Boolean}          true if the element is a multiselect, false if not
 */
const isMultiSelect = element => element.options && element.multiple;

/**
 * Retrieves the selected options from a multi-select as an array.
 * @param  {HTMLOptionsCollection} options  the options for the select
 * @return {Array}                          an array of selected option values
 */
const getSelectValues = options => [].reduce.call(options, (values, option) => {
  return option.selected ? values.concat(option.value) : values;
}, []);

/**
 * Retrieves input data from a form and returns it as a JSON object.
 * @param  {HTMLFormControlsCollection} elements  the form elements
 * @return {Object}                               form data as an object literal
 */
const formToJSON = elements => [].reduce.call(elements, (data, element) => {

  // Make sure the element has the required properties and should be added.
  if (isValidElement(element) && isValidValue(element)) {

    /*
     * Some fields allow for more than one value, so we need to check if this
     * is one of those fields and, if so, store the values as an array.
     */
    if (isCheckbox(element)) {
      data[element.name] = (data[element.name] || []).concat(element.value);
    } else if (isMultiSelect(element)) {
      data[element.name] = getSelectValues(element);
    } else {
      data[element.name] = element.value;
    }
  }

  return data;
}, {});


/**
 * A handler function to prevent default submission and run our custom script.
 * @param  {Event} event  the submit event triggered by the user
 * @return {void}
 */
const handleFormSubmit = event => {
  
  // Stop the form from submitting since we’re handling that with AJAX.
  event.preventDefault();
  // Set/update the form hidden values before exporting the form
  pages[curForm].status  = pages[curForm].isDraft ? "draft" : "completed";
  pages[curForm].pageTimestamp = moment().unix();
  if (curForm === "personalInterviewForm" || curForm=== "firstInstanceDecision") {pages[curForm].setHiddenFieldsValue();}
  
  // Call our function to get the form data.
  var draft = '';
  if (pages[curForm].isDraft) draft = "_draft";
  var data = formToJSON($("#"+pages[curForm].id)[0].elements);
  var fileName =  pages[curForm].filename("json").replace('_personal','').replace('_first_instance_decision','_decision');
  // Use `JSON.stringify()` to make the output valid, human-readable JSON.
  var strData = JSON.stringify(data, null, "  ");
  if (curForm === "optionsPage") { 

    strData = "var userOptions =" + strData; 
    fileName = pages[curForm].filename("js");
  }
  
  download(fileName, strData);
};
// Custom form submit handlers
const form = document.getElementById('qatSubmit');
form.addEventListener('submit', handleFormSubmit);
const formFid = document.getElementById('qatSubmitFid');
formFid.addEventListener('submit', handleFormSubmit);
const formOpt = document.getElementById('qatOptionsForm');
formOpt.addEventListener('submit', function(event) {event.preventDefault();$("#optionsSaveModal").modal();} );
$('#vidInstructions').click(function(event) {event.preventDefault();$("#optionsSaveModal").modal();});
const formOptSave = document.getElementById('saveOptionsBTN');
formOptSave.addEventListener('click', handleFormSubmit);

function download (filename, text) {
  var blob = new Blob([text], {
    type: "text/plain;charset=utf-8;",
  });
  saveAs(blob, filename);
}

$(function() {
  /** Save as draft listener */
  $("#saveFileBTN, #saveFileBTNFid").click(function() {
    if (pages[curForm].isValidated) {
      handleFormSubmit;
    } else {
      if (!(pages[curForm].hasEmptyRequiredFields())) { 
        pages[curForm].isDraft = false;
        handleFormSubmit; 
      } //We can use this for more tests in the future
      else if ((pages[curForm].hasEmptyRequiredFields())) { 
        return;
      }
      else if (pages[curForm].caseFileReference() != ""){ //caseFileReference is mandatory even for drafts
        event.preventDefault();
        $("#draftSaveModal").modal();
      }
    }
  });
  $(document).on('click', "#saveAsDraftBTN,#saveAsDraftBTNFid", function() {
    pages[curForm].isValidated = true;
    pages[curForm].setNoValidate();
    pages[curForm].isDraft = true;
    clickButton('saveFileBTN'+ pages[curForm].suffix);
    pages[curForm].isValidated = false;
    pages[curForm].remNoValidate();
  });
  //When user clicks on cancel, check report validity in order to show the empty required fields
  $("#saveAsDraftCanselBTN").click(function() {
    document.forms[pages[curForm].id].reportValidity()
  });

});

/** Export/print to PDF */
function printPDF() {
  if (pages[curForm].caseFileReference().length === 0 && curForm != "reportPage") {
    document.forms[pages[curForm].id].reportValidity();
    return false;
  } 
  
  // Call function that checks multiple placeholders when loading a json
  if (isEdge)	checkPlaceholderMultiple();

  // The main title of page cover
  $("#reportCoverPageMainTitleLine2")[0].innerText = recuringTrans[curForm].reportCoverPageMainTitleLine2;
  $("#reportCoverPageMainTitleLine3")[0].innerText = recuringTrans[curForm].reportCoverPageMainTitleLine3;
  
  //Add CSS for printing
  var url = "css/html2pdfCustom.css";
  var head = document.getElementsByTagName('head')[0];
  var link = document.createElement('link');
  link.id = "printCssLink";
  link.type = "text/css";
  link.rel = "stylesheet";
  link.href = url;
  
  // Add class before producing the reports pdf
  $(".resultsRep").addClass("offset-xl-2");
  //head.appendChild(link);

  $('#appliedFilters tr').remove();
  $('.chrome-data-fix').remove();
  $('#reportCommentsContainer textarea').hide();
  
  if(reportTable){

    $.each(reportTable.getFilters(true), function( index, val ) {
      var field = val.field;
      var filterDisplayContent = null;
      if(moment(val.value.start, "YYYY-MM-DD", true).isValid() && moment(val.value.end, "YYYY-MM-DD", true).isValid()) {

        filterDisplayContent  = val.value.start + ' - '+ val.value.end
        
      } else if(moment(val.value.start, "YYYY-MM-DD", true).isValid()) {
        filterDisplayContent  = val.value.start + ' -';
      }else if(moment(val.value.end, "YYYY-MM-DD", true).isValid()) {
        filterDisplayContent  =  ' - ' + val.value.end
      } else{
        filterDisplayContent  =  val.value.join(',');
      }
      if(filterDisplayContent)
      $('#appliedFilters').append('<tr id="'+ field +'AppliedFilter"><td>'+ recuringTrans[field] +'</td><td>' + filterDisplayContent +'</td></tr>');


    });
  }

 


  // show which filters have been applied
  if( $('#appliedFilters tr').length > 0) {
    $('.appliedFilters').show();
  }

  //Make sure that the CSS file has been loaded before startin the pdf creation process
  //link.onload = function () {

    var content = '';
    var report_selected = ''


    if($('input[name="selected_report"]:checked').val()=='personal') {
      report_selected = 'personal';
       content = $('#reportCommentsInterviews').val().replace(/\n([ \t]*\n)+/g, '</p><p>')
       .replace('\n', '<br />') ;
    }else {
      report_selected ='first_instance';
      content =  $('#reportCommentsDecisions').val().replace(/\n([ \t]*\n)+/g, '</p><p>')
       .replace('\n', '<br />') ;
    }
    var todaysDate = new Date();

    var js_month = todaysDate.getMonth() + 1;

    var month = String(js_month).padStart(2, '0');
    var day = String(todaysDate.getDate()).padStart(2, '0');
    var year = todaysDate.getFullYear();

    // var minutes  = String(todaysDate.getMinutes()).padStart(2, '0');
    // var hours  = String(todaysDate.getHours()).padStart(2, '0');
    // var seconds = String(todaysDate.getSeconds()).padStart(2, '0');

    var dateTimeFile = year+month+day;//+'_'+hours+minutes+seconds;

    var pdfFileName = '';

    if (curForm === "personalInterviewForm" || curForm=== "firstInstanceDecision") {
      pdfFileName = 'interview_report_' + dateTimeFile+'.pdf';
      if(curForm=== "firstInstanceDecision") {
        pdfFileName = 'decision_report_' + dateTimeFile+'.pdf';
      }
      pdfFileName = pages[curForm].caseFileReference() + '_' + pdfFileName;
    }else{
      pdfFileName = 'decisions_' + dateTimeFile+'.pdf';
      if($('input[name="selected_report"]:checked').val()=='personal') {
        pdfFileName = 'interviews_' + dateTimeFile+'.pdf';
      }
      pdfFileName = 'Report_' + pdfFileName;
    }


    $('input,textarea', 'body').each(function(x,y){

      if($(this).val().length<=0 || $(this).attr('type')=='hidden' || $(this).attr('type')=='button'|| $(this).attr('type')=='submit') {
        return;
      }

      if($(this).attr('id')=='reportCommentsInterviews' || $(this).attr('id')=='reportCommentsDecisions' ){
        return;
      }

      if($(this).prop("tagName")=='TEXTAREA' ) {
      
        $(this).parent().append('<p class="chrome-data-fix chrome-data-fix-full-width">' + $(this).val() + '</p>' )
      }else{
        $(this).parent().append('<span class="chrome-data-fix">' + $(this).val() + '</span>' )
      }

    });


    var source = window.document.getElementsByTagName("body")[0];
    var opt = {
      margin:       [0,-1.5,0,0],
      filename:     pdfFileName,
      image:        { type: 'jpeg', quality: 1 },
      html2canvas:  { scale: 1 },
      jsPDF:        { unit: 'in', format: 'a4', orientation: 'portrait'}
    };
    
    $('title').text(pdfFileName);

    $('#r-comments').html(content);
    var textLength = 0;
    var commentsPageCount = 0;
    var commentsPageLength = 1200;
    $( "p" , '#r-comments' ).each(function( index ) {

      textLength += $( this ).text().length;
      if(textLength>commentsPageLength){
        $( '<div class="html2pdf__page-break"></div><br /><br />' ).insertAfter( $( this ) );
        textLength = 0;
        commentsPageLength = 3100;
      }
    });
    
 
    print();
 
    return;
    
      //After the PDF has been created (successfully or not), remove the print CSS
      html2pdf(source,opt).then(function() {
        // re hide applied filters
        $('.appliedFilters').hide();
        $('#printCssLink').remove();
        $('.chrome-data-fix').remove();
        // Remove class after producing the reports pdf
        $(".resultsRep").removeClass("offset-xl-2");

        if(report_selected=='personal') {
          $('input[name="selected_report"][value="personal"]').prop('checked', true);
        } else if(tableDataFirstInstanceDecision.length<=0 && tableDataPersonalInterviewForm.length>0) {
          $('input[name="selected_report"][value="first_instance"]').prop('checked', true);
        }

      });


 // }
  
};


window.onafterprint = function(){
  console.log("Printing completed...");

  if($('input[name="selected_report"]:checked').val()=='personal') {
    report_selected = 'personal';
  }else {
    report_selected ='first_instance';
  }
 

          // re hide applied filters
          $('.appliedFilters').hide();
          $('#printCssLink').remove();
          $('.chrome-data-fix').remove();
          
          // Remove class after producing the reports pdf
          $(".resultsRep").removeClass("offset-xl-2");
  
          if(report_selected=='personal') {
            $('#reportCommentsInterviews').show();
            $('input[name="selected_report"][value="personal"]').prop('checked', true);
          } else if(tableDataFirstInstanceDecision.length<=0 && tableDataPersonalInterviewForm.length>0) {
            $('#reportCommentsDecisions').show();
            $('input[name="selected_report"][value="first_instance"]').prop('checked', true);
          }   
}

/** Export Report filter values to JSON*/
function exportFilterValues(data) {
  var fileName = pages[curForm].filename("json");
  // Use `JSON.stringify()` to make the output valid, human-readable JSON.
  download(fileName, JSON.stringify(data, null, "  "));
}
